/***************************************************************************
                          exserdoc.h  -  description
                             -------------------
    begin                : Tue Jun 26 09:42:14 PDT 2001
    copyright            : (C) 2001 by root
    email                : root@localhost.localdomain
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef EXSERDOC_H
#define EXSERDOC_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for QT
#include <qobject.h>
#include <qstring.h>
#include <qlist.h>

#include "serparmdlg.h"

// forward declaration of the Exser classes
class ExserView;

typedef struct exser_read_reg {
        int reg;
        int regvalue;
};

/**	ExserDoc provides a document object for a document-view model.
  *
  * The ExserDoc class provides a document object that can be used in conjunction with the classes
  * ExserApp and ExserView to create a document-view model for MDI (Multiple Document Interface)
  * Qt 2.1 applications based on QApplication and QMainWindow as main classes and QWorkspace as MDI manager widget.
  * Thereby, the document object is created by the ExserApp instance (and kept in a document list) and contains
  * the document structure with the according methods for manipulating the document
  * data by ExserView objects. Also, ExserDoc contains the methods for serialization of the document data
  * from and to files.
  * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team. 	
  * @version KDevelop version 1.1 code generation
  */
class ExserDoc : public QObject
{
  Q_OBJECT

  friend ExserView;

  public:
    /** Constructor for the fileclass of the application */
    ExserDoc();
    /** Destructor for the fileclass of the application */
    ~ExserDoc();

    /** adds a view to the document which represents the document contents. Usually this is your main view. */
    void addView(ExserView *view);
    /** removes a view from the list of currently connected views */
    void removeView(ExserView *view);
		/** gets called if a view is removed or added */
		void changedViewList();
    /** returns the first view instance */
		ExserView* firstView(){ return pViewList->first(); };
		/** returns true, if the requested view is the last view of the document */
    bool isLastView();
    /** This method gets called when the user is about to close a frame window. It checks, if more than one view
    	* is connected to the document (then the frame can be closed), if pFrame is the last view and the document is
    	* modified, the user gets asked if he wants to save the document.
    	*/
		bool canCloseFrame(ExserView* pFrame);
    /** sets the modified flag for the document after a modifying action on the view connected to the document.*/
    void setModified(bool _m=true){ modified=_m; };
    /** returns if the document is modified or not. Use this to determine if your document needs saving by the user on closing.*/
    bool isModified(){ return modified; };
    /** deletes the document's contents */
    void deleteContents();
    /** initializes the document generally */
    bool newDocument();
    /** closes the acutal document */
    void closeDocument();
    /** loads the document by filename and format and emits the updateViews() signal */
    bool openDocument(const QString &filename, const char *format=0);
    /** saves the document under filename and format.*/	
    bool saveDocument(const QString &filename, const char *format=0);
    /** sets the path to the file connected with the document */
    void setPathName(const QString &name);
    /** returns the pathname of the current document file*/
    const QString& pathName() const;

    /** sets the filename of the document */
    void setTitle(const QString &title);
    /** returns the title of the document */
    const QString& title() const;
	
    /** Get the serial port fd */
    int getSerfd() { return serfd; };
		
    /** Get the read buffer */
    char *getRdBuf() { return rdBuf; };
		
		/** Set the serial port with the parameters */
		bool SetSerialParm();
		
		/** Set serial port parameters dialog box */
		void SetSerParm();

		/* set serial port test parameters */
		void SetSerTest();
		
		/** Run the selected test */
		void SerRunTest();
		
		/** Stop the running test */
		void SerStopTest();
		
		/* XR17C158 initialization function */
		void XR17C158Init();

		/* XR ST16C550 initialization function */
		void ST16C550Init();
		
		/* XR ST16C650 initialization function */
		void ST16C650Init();
		
		/* XR XR16C750 initialization function */
		void XR16C750Init();
		
		/* XR16C850 init function */
		void XR16C850Init();

		/* set the serial port parameters dialog box */
		void STD8250SetParm( SerParmDlg* dlg );
		void ST550SetParm( SerParmDlg* dlg );
		void ST650SetParm( SerParmDlg* dlg );
		void XR750SetParm( SerParmDlg* dlg );
		void XR850SetParm( SerParmDlg* dlg );

	protected:
	void  timerEvent( QTimerEvent * );		
	
	private slots:
    /* Comread slot */
		void ComRead();

  public slots:
    /** calls repaint() on all views connected to the document object and is called by the view by which the document has been changed.
     * As this view normally repaints itself, it is excluded from the paintEvent.
     */
    void updateAllViews(ExserView *sender);
 	
  private:
    /** the modified flag of the current document */
    bool modified;
    QString m_title;
    QString m_filename;
    /** the list of the views currently connected to the document */
    QList<ExserView> *pViewList;	

    /** Serial port file descriptor */
    int serfd;
    char *rdBuf;

    char *TxBuf, *RxBuf;

    /** number of bytes in the read buffer */
    int noBytes;

    /** Serial port parameters */
    Q_INT32 baudrate, databits, parity, stopbits, flowcontrol;
    Q_INT32 dtrhystflg, rtshystflg, dtrhyst, rtshyst;
    Q_INT32 txfifoflg, rxfifoflg, txtriglvl, rxtriglvl;
    Q_INT32 intloopb, autorsflg, PatLen;
    Q_INT32	testpat, packerr;
    Q_INT32 StopOnErrFlg, ContFlg, NoPackFlg, NoPack;
    Q_INT32	UARTType;

    int TimerID, TestingFlg, noTx, testfd, TxBytes, RxBytes;

    QString TestFlName;

};

#endif // EXSERDOC_H
